/*
* Copyright (c) 2004 Sun Microsystems, Inc. All Rights Reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*
* -Redistribution of source code must retain the above copyright notice, this
*  list of conditions and the following disclaimer.
*
* -Redistribution in binary form must reproduce the above copyright notice,
*  this list of conditions and the following disclaimer in the documentation
*  and/or other materials provided with the distribution.
*
* Neither the name of Sun Microsystems, Inc. or the names of contributors may
* be used to endorse or promote products derived from this software without
* specific prior written permission.
*
* This software is provided "AS IS," without a warranty of any kind. ALL
* EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING
* ANY IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* OR NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN MIDROSYSTEMS, INC. ("SUN")
* AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE
* AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS
* DERIVATIVES. IN NO EVENT WILL SUN OR ITS LICENSORS BE LIABLE FOR ANY LOST
* REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,
* INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY
* OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE,
* EVEN IF SUN HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
*
* You acknowledge that this software is not designed, licensed or intended
* for use in the design, construction, operation or maintenance of any
* nuclear facility.
*/
package ru.ezsk.demo.graphLayout;

import java.applet.AudioClip;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.util.StringTokenizer;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ActionMap;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonModel;
import javax.swing.InputMap;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JComponent;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JRootPane;
import javax.swing.JSlider;
import javax.swing.JToggleButton;
import javax.swing.KeyStroke;
import javax.swing.UIManager;
import javax.swing.JLabel;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;
import javax.swing.border.EtchedBorder;
import javax.swing.border.BevelBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * @version $Revision: 62 $ $Author: beholder $
 *          $Date: 2005-11-02 16:06:21 +0300 (Ср, 02 ноя 2005) $
 */
public class Application
{
    //=========================================================== Class Fields

    private static final String SCRAMBLE_SOUND = "audio/computer.wav";

    private static final String SHAKE_SOUND = "audio/gong.wav";

    private static final String RANDOM_SOUND = "audio/drip.wav";

    //================================================================= Fields

    private GraphPanel graphPanel;

    private boolean initialized;

    private AudioClip scrambleClip;

    private AudioClip shakeClip;

    private AudioClip randomClip;

    private JToggleButton addNodeBtn;

    private boolean quiet;

    private JToggleButton createEdgeBtn;

    private AbstractAction renameNodeAction;

    private AbstractAction deleteNodeAction;

    private ButtonModel hideNamesChkModel;

    private ButtonModel showStressChkModel;

    private final Context context;

    private ButtonModel animateChkMdl;

    private JLabel statusLbl;

    //=========================================================== Constructors

    public Application(Context context) {
        this.context = context;
    }

    //================================================================ Methods

    public void init() {
        setLookAndFeel();

        final JPanel content = new JPanel(new BorderLayout());
        context.getRootPane().setContentPane(content);

        createActions();

        graphPanel = new GraphPanel(this);
        final Dimension size = new Dimension(400, 400);
        graphPanel.setMinimumSize(size);
        graphPanel.setPreferredSize(size);
        //graphPanel.setMaximumSize(size);
        graphPanel.setAlignmentY(JComponent.TOP_ALIGNMENT);
        content.add(BorderLayout.CENTER, graphPanel);

        statusLbl = new JLabel("Graph Layout Demo");
        content.add(BorderLayout.SOUTH, statusLbl);
        statusLbl.setBorder(new BevelBorder(BevelBorder.RAISED));

        readParameters();

        createMenuBar();

        createControlPanel(content);
    }

    private void showStatus(String status) {
        statusLbl.setText(status);
        //context.showStatus(status);
    }

    private void createActions() {
        deleteNodeAction = new AbstractAction("Delete") {
            public void actionPerformed(ActionEvent e) {
                graphPanel.deleteSelectedNode();
            }
        };
        deleteNodeAction.setEnabled(false);
        deleteNodeAction.putValue(Action.ACCELERATOR_KEY,
                KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, 0));
        deleteNodeAction.putValue(Action.MNEMONIC_KEY, (int) 'D');

        renameNodeAction = new AbstractAction("Rename") {
            public void actionPerformed(ActionEvent e) {
                graphPanel.renameNode();
            }
        };
        renameNodeAction.setEnabled(false);
        renameNodeAction.putValue(Action.MNEMONIC_KEY, (int) 'R');
        renameNodeAction.putValue(Action.ACCELERATOR_KEY,
                KeyStroke.getKeyStroke(KeyEvent.VK_F2, 0));
        renameNodeAction.putValue(Action.ACTION_COMMAND_KEY, "renameNode");
    }

    private void createMenuBar() {
        final JMenuBar menuBar = new JMenuBar();
        context.getRootPane().setJMenuBar(menuBar);

        //=========
        final JMenu nodesMenu = new JMenu("Node");
        nodesMenu.setMnemonic('N');
        menuBar.add(nodesMenu);

        final JMenuItem deleteNodeItem = new JMenuItem(deleteNodeAction);
        nodesMenu.add(deleteNodeItem);

        final JMenuItem deleteAllItem = new JMenuItem("Delete All");
        deleteAllItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                graphPanel.clear();
            }
        });
        deleteAllItem.setMnemonic('A');
        nodesMenu.add(deleteAllItem);

        //=========
        final JMenu edgesMenu = new JMenu("Edge");
        edgesMenu.setMnemonic('E');
        menuBar.add(edgesMenu);

        final JMenuItem createEdgeItem = new JMenuItem("Create");
        createEdgeItem.setMnemonic('C');
        edgesMenu.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
            }
        });
        edgesMenu.add(createEdgeItem);

        //=========
        final JMenu optionsMenu = new JMenu("Options");
        optionsMenu.setMnemonic('O');
        menuBar.add(optionsMenu);

        final JCheckBoxMenuItem animateItm = new JCheckBoxMenuItem("Animate");
        animateItm.setMnemonic('A');
        optionsMenu.add(animateItm);
        animateChkMdl = animateItm.getModel();
        animateChkMdl.setSelected(true);
        animateChkMdl.addItemListener(new XItemListener() {
            protected void stateChanged(boolean selected) {
                if (selected) {
                    graphPanel.start();
                } else {
                    graphPanel.stop();
                }
            }
        });

        final JCheckBoxMenuItem showStressItem =
                new JCheckBoxMenuItem("Show stress");
        showStressItem.setMnemonic('s');
        showStressItem.setDisplayedMnemonicIndex(5);
        optionsMenu.add(showStressItem);
        showStressChkModel = showStressItem.getModel();
        showStressChkModel.addItemListener(new XItemListener() {
            protected void stateChanged(boolean selected) {
                graphPanel.showStress = selected;
            }
        });

        final JCheckBoxMenuItem hideNamesItem =
                new JCheckBoxMenuItem("Hide names");
        hideNamesItem.setMnemonic('n');
        optionsMenu.add(hideNamesItem);
        hideNamesChkModel = hideNamesItem.getModel();
        hideNamesChkModel.addItemListener(new XItemListener() {
            protected void stateChanged(boolean selected) {
                graphPanel.setHideNames(selected);
            }
        });
    }

    private void setLookAndFeel() {
        //noinspection OverlyBroadCatchBlock
        try {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        } catch (Exception e) {
            // do nothing
        }
    }

    private void createControlPanel(JPanel content) {
        final JPanel controlPanel = new JPanel(null);
        controlPanel.setLayout(new BoxLayout(controlPanel, BoxLayout.Y_AXIS));
        controlPanel.setAlignmentY(JComponent.TOP_ALIGNMENT);
        final EmptyBorder eb = new EmptyBorder(5, 5, 5, 5);
        final Border brd = XBorders.compound(eb, new EtchedBorder(), eb);
        controlPanel.setBorder(brd);

        createButtons(controlPanel);
        controlPanel.add(Box.createVerticalStrut(5));

        final JCheckBox stressCheckBox = new JCheckBox("Stress");
        stressCheckBox.setModel(showStressChkModel);
        controlPanel.add(stressCheckBox);

        final JCheckBox hideNamesChk = new JCheckBox("Hide names");
        hideNamesChk.setModel(hideNamesChkModel);
        controlPanel.add(hideNamesChk);

        createJitterControl(controlPanel);

        controlPanel.add(Box.createVerticalStrut(5));

        createNodeButtons(controlPanel);

        controlPanel.add(Box.createVerticalGlue());

        final JCheckBox animateChk = new JCheckBox("Animate");
        animateChk.setModel(animateChkMdl);
        controlPanel.add(animateChk);

        final JCheckBox quietChk = new JCheckBox("Queit");
        quietChk.setMnemonic('Q');
        quietChk.setSelected(quiet);
        quietChk.addItemListener(new XItemListener() {
            protected void stateChanged(boolean selected) {
                quiet = selected;
            }
        });
        controlPanel.add(quietChk);

        content.add(BorderLayout.EAST, controlPanel);
    }

    private void createNodeButtons(JPanel controlPanel) {
        final JPanel panel = new JPanel(new GridLayout(0, 1, 0, 2));

        addNodeBtn = new JToggleButton("Add node");
        addNodeBtn.setMnemonic('A');
        addNodeBtn.addItemListener(new XItemListener() {
            protected void stateChanged(boolean selected) {
                graphPanel.addNewNodeMode = selected;
            }
        });
        panel.add(addNodeBtn);

        final JButton deleteNodeBtn = new JButton(deleteNodeAction);
        //deleteNodeBtn.setMnemonic('D');
        //deleteNodeBtn.addActionListener(deleteNodeAction);
        panel.add(deleteNodeBtn);

        createRenameNodeButton(panel);

        panel.add(Box.createVerticalStrut(15));

        createCreateEdgeButton(panel);

        panel.setMaximumSize(panel.getMinimumSize());
        panel.setAlignmentX(JComponent.LEFT_ALIGNMENT);
        controlPanel.add(panel);
    }

    private void createRenameNodeButton(JPanel panel) {
        final JButton renameNodeBtn = new JButton(renameNodeAction);
        //renameNodeBtn.setMnemonic('R');

        panel.add(renameNodeBtn);

        //registerKeyboardAction(renameNodeAction);
    }

    @SuppressWarnings({"UNUSED_SYMBOL"})
    private void registerKeyboardAction(Action action) {
        final String actionCommand =
                (String) action.getValue(Action.ACTION_COMMAND_KEY);
        final JRootPane rootPane = context.getRootPane();

        final InputMap inputMap = rootPane.getInputMap(
                JComponent.WHEN_IN_FOCUSED_WINDOW);
        final KeyStroke keyStroke =
                (KeyStroke) action.getValue(Action.ACCELERATOR_KEY);
        inputMap.put(keyStroke, actionCommand);

        final ActionMap actionMap = rootPane.getActionMap();
        actionMap.put(actionCommand, action);
    }

    private void createCreateEdgeButton(JPanel panel) {
        createEdgeBtn = new JToggleButton("Create edge");
        createEdgeBtn.setMultiClickThreshhold(100);
        createEdgeBtn.setMnemonic('e');
        createEdgeBtn.setDisplayedMnemonicIndex(7);
        createEdgeBtn.setEnabled(false);
        createEdgeBtn.addItemListener(new XItemListener() {
            protected void stateChanged(boolean selected) {
                if (selected) {
                    showStatus("Select end node for edge");
                } else {
                    showStatus("");
                }
                graphPanel.createNewEdgeMode = selected;
            }
        });
        panel.add(createEdgeBtn);
    }

    private void createJitterControl(JPanel controlPanel) {
        final JSlider slider = new JSlider(JSlider.HORIZONTAL, 0, 100, 5);

        final JCheckBox jitterChk = new JCheckBox("Random jitter");
        jitterChk.setMnemonic('J');
        jitterChk.addItemListener(new XItemListener() {
            protected void stateChanged(boolean enabled) {
                graphPanel.setJitterEnabled(enabled);
                slider.setEnabled(enabled);
            }
        });
        controlPanel.add(jitterChk);

        slider.setEnabled(false);
        slider.setMinorTickSpacing(5);
        slider.setMajorTickSpacing(25);
        slider.setPaintTicks(true);
        slider.setPaintLabels(true);
        slider.putClientProperty("JSlider.isFilled", Boolean.TRUE);
        slider.addChangeListener(new ChangeListener() {
            public void stateChanged(ChangeEvent e) {
                final int value = slider.getValue();
                showStatus("Jitter randomness: " + value);
                graphPanel.setJitterRandomness(value);
            }
        });
        slider.setAlignmentX(JComponent.LEFT_ALIGNMENT);
        slider.setPreferredSize(
                new Dimension(120, slider.getPreferredSize().height));
        controlPanel.add(slider);
    }

    private void createButtons(JPanel controlPanel) {
        final JPanel buttonsPanel = new JPanel(new GridLayout(0, 1, 0, 2));
        buttonsPanel.setAlignmentX(JComponent.LEFT_ALIGNMENT);

        final JButton scrambleButton = new JButton("Scramble");
        scrambleButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                playScrambleSound();
                graphPanel.scramble();
            }
        });
        buttonsPanel.add(scrambleButton);

        final JButton shakeButton = new JButton("Shake");
        shakeButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                playShakeSound();
                graphPanel.shake();
            }
        });
        buttonsPanel.add(shakeButton);

        final JButton freezeButton = new JButton("Freeze!");
        freezeButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                graphPanel.freeze();
            }
        });
        buttonsPanel.add(freezeButton);

        buttonsPanel.setMaximumSize(buttonsPanel.getPreferredSize());
        controlPanel.add(buttonsPanel);
    }

    private void readParameters() {
        final String edges = context.getParameter("edges");
        for (final StringTokenizer t = new StringTokenizer(edges, ", \t\n\r");
             t.hasMoreTokens();)
        {
            String str = t.nextToken();
            final int dashPos = str.indexOf('-');
            if (dashPos > 0) {
                final int length;
                final int slashPos = str.indexOf('/');
                if (slashPos > 0) {
                    length = Integer.parseInt(str.substring(slashPos + 1));
                    str = str.substring(0, slashPos);
                } else {
                    length = Edge.DEFAULT_LENGTH;
                }
                final String fromNodeName = str.substring(0, dashPos);
                final String toNodeName   = str.substring(dashPos + 1);
                graphPanel.addEdge(fromNodeName, toNodeName, length);
            }
        }

        final String quietStr = context.getParameter("quiet");
        if (quietStr != null) {
            quiet = Boolean.parseBoolean(quietStr);
        }
    }

    public void destroy() {
        context.getRootPane().getContentPane().removeAll();
    }

    public void start() {
        if (!initialized) {
            final String center = context.getParameter("center");
            if (center != null) {
                final Dimension d = graphPanel.getSize();
                final Node n = graphPanel.findNode(center);
                n.center(d);
                n.setFixed(true);
            }
            graphPanel.scramble();
            initialized = true;
        }
        if (!animateChkMdl.isSelected()) {
            return;
        }
        graphPanel.start();
    }

    public void stop() {
        if (!animateChkMdl.isSelected()) {
            return;
        }
        graphPanel.stop();
    }

    //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    public void playScrambleSound() {
        if (quiet) {
            return;
        }
        if (scrambleClip == null) {
            scrambleClip = context.getAudioClip(SCRAMBLE_SOUND);
        }
        scrambleClip.play();
    }

    public void playShakeSound() {
        if (quiet) {
            return;
        }
        if (shakeClip == null) {
            shakeClip = context.getAudioClip(SHAKE_SOUND);
        }
        shakeClip.play();
    }

    public void playRandomSound() {
        if (quiet) {
            return;
        }
        if (randomClip == null) {
            randomClip = context.getAudioClip(RANDOM_SOUND);
        }
        randomClip.play();
    }

    public void notifyNodeAdded() {
        addNodeBtn.setSelected(false);
    }

    public void notifyNodeSelection(boolean selected) {
        deleteNodeAction.setEnabled(selected);
        createEdgeBtn.setEnabled(selected);
        renameNodeAction.setEnabled(selected);
    }

    public void notifyEdgeCreated(boolean success) {
        createEdgeBtn.setSelected(false);
        //createEdgeBtn.setEnabled(false);
    }
}
