package ru.ezsk.demo.graphLayout;

import java.awt.Color;
import java.awt.Graphics2D;

/**
 * @version $Revision: 48 $ $Author: beholder $
 *          $Date: 2005-10-27 15:10:46 +0400 (Чт, 27 окт 2005) $
 */
public class Edge
{
    //=========================================================== Class Fields

    private static final Color EDGE_COLOR = Color.BLACK;

    private static final Color STRESS_COLOR = Color.DARK_GRAY;

    private static final Color ARC_COLOR1 = Color.BLACK;

    private static final Color ARC_COLOR2 = Color.PINK;

    private static final Color ARC_COLOR3 = Color.RED;

    public static final int DEFAULT_LENGTH = 50;

    //========================================================== Class Methods

    private static Color arcColor(int length) {
        if (length < 10) {
            return ARC_COLOR1;
        } else if (length < 20) {
            return ARC_COLOR2;
        } else {
            return ARC_COLOR3;
        }
    }

    //================================================================= Fields

    private Node from;

    private Node to;

    private double length;

    //=========================================================== Constructors

    public Edge(Node from, Node to, double length) {
        if (from == to) {
            throw new IllegalArgumentException("Edge have same endpoints");
        }
        this.from = from;
        this.to = to;
        this.length = length;
    }

    //================================================================ Methods

    public Node getFrom() {
        return from;
    }

    public Node getTo() {
        return to;
    }

    public void relax() {
        final double vx = to.x - from.x;
        final double vy = to.y - from.y;
        double len = Math.sqrt(XMath.sqr(vx) + XMath.sqr(vy));
        if (len < 0.0001) {
            len = 0.0001;
        }
        final double f = (length - len) / (len * 3);
        final double dx = f * vx;
        final double dy = f * vy;

        to.accelerate(dx, dy);
        from.accelerate(-dx, -dy);
    }

    public void paint(Graphics2D g, boolean showStress) {
        final int x1 = (int) from.x;
        final int y1 = (int) from.y;
        final int x2 = (int) to.x;
        final int y2 = (int) to.y;
        final int len = (int) Math.abs(XMath.distance(x1, x2, y1, y2) - length);
        g.setColor(arcColor(len));
        g.drawLine(x1, y1, x2, y2);
        if (showStress) {
            final String lbl = String.valueOf(len);
            g.setColor(STRESS_COLOR);
            g.drawString(lbl, x1 + (x2 - x1) / 2, y1 + (y2 - y1) / 2);
            g.setColor(EDGE_COLOR);
        }
    }

    public boolean isConnectedTo(Node node) {
        return (from == node) || (to == node);
    }

    public boolean endpointsEquals(Node node1, Node node2) {
        return ((from == node1) && (to   == node2)) ||
               ((to   == node1) && (from == node2));
    }
}
