package ru.ezsk.demo.graphLayout;

import static ru.ezsk.demo.graphLayout.XMath.sqr;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.RenderingHints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import static java.lang.Math.sqrt;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Random;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.Timer;
import javax.swing.event.MouseInputAdapter;

/**
 * @version $Revision: 62 $ $Author: beholder $
 *          $Date: 2005-11-02 16:06:21 +0300 (Ср, 02 ноя 2005) $
 */
public class GraphPanel extends JPanel
{
    //=========================================================== Class Fields

    private static final int SHAKE_OFFSET = 50;

    private static final int X_PAD = 10;

    private static final int Y_PAD = 10;

    private static final int BEST_DISTANCE = 100;

    private static final int DELAY = 40;

    //================================================================= Fields

    private final Application application;

    private final List<Edge> edgesList = new ArrayList<Edge>(20);

    boolean showStress;

    private boolean jitterEnabled;

    private Node pick;

    private Image offScreen;

    private Dimension offScreenSize;

    private Graphics2D offGraphics;

    private final Map<String, Node> nodesMap = new HashMap<String, Node>(20);

    // List is kept only for random node retreival.
    private final List<Node> nodesList = new ArrayList<Node>(20);

    private Timer timer;

    private final Random rng = new Random();

    private int jitterRandomness = 3;

    public boolean addNewNodeMode;

    public boolean createNewEdgeMode;

    private boolean hideNames;

    private int cnt;

    private Thread thread;

    private volatile boolean running;

    private Object lock = new Object();

    //=========================================================== Constructors

    GraphPanel(Application application) {
        super(null, false);
        setFont(new Font("Tahoma", Font.PLAIN, 11));
        this.application = application;

        final MouseHandler mouseHandler = new MouseHandler();
        addMouseListener(mouseHandler);
        addMouseMotionListener(mouseHandler);
    }

    //================================================================ Methods

    public Node findNode(String label) {
        Node node = nodesMap.get(label);
        if (node == null) {
            node = addNode(label);
        }
        return node;
    }

    private Node addNode(String label) {
        final Node node = new Node(label);
        nodesMap.put(label, node);
        nodesList.add(node);
        return node;
    }

    public void addEdge(String from, String to, int length) {
        final Node fromNode = findNode(from);
        final Node toNode   = findNode(to);
        final Edge edge = new Edge(fromNode, toNode, length);
        edgesList.add(edge);
    }

    private Collection<Node> nodes() {
        return nodesList;
    }

    public void start() {
        if (timer != null) {
        //if (running) {
            System.err.println("already started");
            return;
        }
        final ActionListener actionListener = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                tick();
                step();
            }
        };
        //thread = new Thread("Animation thread") {
        //    public void run() {
        //        animate();
        //    }
        //
        //};
        //running = true;
        //thread.start();
        timer = new Timer(DELAY, actionListener);
        timer.setRepeats(true);
        timer.setCoalesce(false);
        timer.start();

    }

    //private void animate() {
    //    while (running) {
    //        //try {
    //        //    Thread.sleep(DELAY);
    //        //} catch (InterruptedException e) {
    //        //    e.printStackTrace();
    //        //}
    //        synchronized (lock) {
    //            try {
    //                lock.wait(DELAY / 2);
    //            } catch (InterruptedException e) {
    //                e.printStackTrace();
    //            }
    //        }
    //        try {
    //            EventQueue.invokeAndWait(new Runnable() {
    //                public void run() {
    //                    step();
    //                }
    //            });
    //        } catch (InterruptedException e) {
    //            e.printStackTrace();
    //        } catch (InvocationTargetException e) {
    //            e.printStackTrace();
    //        }
    //    }
    //}

    public void stop() {
        if (timer == null) {
        //if (!running) {
            System.err.println("not started");
            return;
        }
        timer.stop();
        timer = null;
        //running = false;
        //thread = null;
    }

    private void step() {
        relax();
        if (jitterEnabled && (rng.nextInt(100) < jitterRandomness)) {
            final int idx = rng.nextInt(nodesList.size());
            final Node n = nodesList.get(idx);
            n.shake(rng, SHAKE_OFFSET);
            application.playRandomSound();
        }
        repaint(DELAY);
    }

    private void tick() {
        System.out.print('.');
        System.out.flush();
        if (++cnt > 50) {
            System.out.println(System.currentTimeMillis());
            cnt = 0;
        }
    }

    private void relaxEdges() {
        for (final Edge edge : edgesList) {
            edge.relax();
        }
    }

    private void relaxNodes() {
        for (final Node n1 : nodes()) {
            double dx = 0;
            double dy = 0;

            for (final Node n2 : nodes()) {
                if (n1 == n2) {
                    continue;
                }
                final double xDiff = n1.x - n2.x;
                final double yDiff = n1.y - n2.y;
                final double distSqr = sqr(xDiff) + sqr(yDiff);
                if (distSqr < 0.0001) {
                    dx += rng.nextDouble() - 0.5;
                    dy += rng.nextDouble() - 0.5;
                } else if (distSqr < sqr(BEST_DISTANCE)) {
                    dx += xDiff / distSqr;
                    dy += yDiff / distSqr;
                }
            }
            final double speedSqr = sqr(dx) + sqr(dy);
            if (speedSqr > 0) {
                final double speed = sqrt(speedSqr) / 2;
                n1.dx += dx / speed;
                n1.dy += dy / speed;
            }
        }
    }

    void relax() {
        relaxEdges();

        relaxNodes();

        final Dimension size = getSize();
        for (final Node node : nodes()) {
            node.move(size);
        }
        repaint();
    }

    public void paintComponent(Graphics g) {
        final Dimension d = getSize();
        if ((offScreen == null) ||
            (d.width != offScreenSize.width) ||
            (d.height != offScreenSize.height))
        {
            offScreen = createImage(d.width, d.height);
            offScreenSize = d;
            if (offGraphics != null) {
                offGraphics.dispose();
            }
            offGraphics = (Graphics2D) offScreen.getGraphics();
            offGraphics.setFont(getFont());
        }

        antialiasOn(offGraphics);
        offGraphics.setRenderingHint(RenderingHints.KEY_RENDERING,
                                     RenderingHints.VALUE_RENDER_QUALITY);
        offGraphics.setRenderingHint(RenderingHints.KEY_STROKE_CONTROL,
                                     RenderingHints.VALUE_STROKE_PURE);

        offGraphics.setColor(getBackground());
        offGraphics.fillRect(0, 0, d.width, d.height);
        for (final Edge edge : edgesList) {
            edge.paint(offGraphics, showStress);
        }

        final Node selectedNode = getSelectedNode();
        for (final Node node : nodes()) {
            node.paint(offGraphics, (node == selectedNode), hideNames);
        }
        g.drawImage(offScreen, 0, 0, null);
    }

    private static void antialiasOn(Graphics2D graphics) {
        graphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                                  RenderingHints.VALUE_ANTIALIAS_ON);
    }

    @SuppressWarnings({"UNUSED_SYMBOL"})
    private static void antialiasOff(Graphics2D graphics) {
        graphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                                  RenderingHints.VALUE_ANTIALIAS_OFF);
    }

    //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    public void scramble() {
        final Dimension size = getSize();
        for (final Node node : nodes()) {
            node.scramble(rng, size, X_PAD, Y_PAD);
        }
    }

    public void shake() {
        for (final Node node : nodes()) {
            node.shake(rng, SHAKE_OFFSET);
        }
    }

    public void freeze() {
        for (final Node node : nodes()) {
            node.freeze();
        }
    }

    private void moveAll(int dx, int dy) {
        for (final Node node : nodes()) {
            node.move(dx, dy);
        }
    }

    //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    public void clear() {
        setSelectedNode(null);
        nodesList.clear();
        nodesMap.clear();
        edgesList.clear();
        repaint();
    }

    private Node findNodeAt(int x, int y) {
        for (final Node node : nodes()) {
            if (node.contains(x, y)) {
                return node;
            }
        }
        return null;
    }

    private void addNewNode(int x, int y) {
        int suffix = nodesList.size();
        String label;
        while (true) {
            label = "n" + suffix;
            if (!nodesMap.containsKey(label)) {
                break;
            }
            suffix++;
        }
        final Node node = new Node(x, y, label);
        nodesList.add(node);
        nodesMap.put(label, node);
        setSelectedNode(node);
        addNewNodeMode = false;
        application.notifyNodeAdded();
    }

    private Node getSelectedNode() {
        return pick;
    }

    private void setSelectedNode(Node node) {
        pick = node;
        application.notifyNodeSelection(node != null);
    }

    public void deleteSelectedNode() {
        final Node node = getSelectedNode();
        if (node == null) {
            return;
        }
        for (Iterator<Edge> iter = edgesList.iterator(); iter.hasNext();) {
            final Edge edge = iter.next();
            if (edge.isConnectedTo(node)) {
                iter.remove();
            }
        }
        nodesList.remove(node);
        nodesMap.remove(node.getLabel());
        setSelectedNode(null);
        repaint();
    }

    public void renameNode() {
        final Node node = getSelectedNode();
        if (node == null) {
            return;
        }
        final String label = node.getLabel();
        final String result =
                JOptionPane.showInputDialog(this, "Rename node", label);
        if ((result != null) && (result.length() > 0) &&
            (!label.equals(result)))
        {
            node.setLabel(result);
            repaint();
        }
    }

    private void addNewEdge(int x, int y) {
        final Node node1 = getSelectedNode();
        if (node1 == null) {
            application.notifyEdgeCreated(false);
            return;
        }
        final Node node2 = findNodeAt(x, y);
        if (node2 == null) {
            //setSelectedNode(null);
            application.notifyEdgeCreated(false);
            return;
        }
        if (node1 == node2) {
            return;
        }
        for (final Edge edge : edgesList) {
            if (edge.endpointsEquals(node1, node2)) {
                return;
            }
        }
        final Edge edge = new Edge(node1, node2, Edge.DEFAULT_LENGTH);
        edgesList.add(edge);
        //setSelectedNode(null);
        application.notifyEdgeCreated(true);
        createNewEdgeMode = false;
    }

    //============================================================= Properties

    public void setJitterRandomness(int jitterRandomness) {
        this.jitterRandomness = jitterRandomness;
    }

    public boolean isJitterEnabled() {
        return jitterEnabled;
    }

    public void setJitterEnabled(boolean jitterEnabled) {
        this.jitterEnabled = jitterEnabled;
    }

    public void setHideNames(boolean hideNames) {
        this.hideNames = hideNames;
        repaint();
    }

    //========================================================== Inner Classes

    class MouseHandler extends MouseInputAdapter
    {
        private static final int MOVE_MASK =
                MouseEvent.CTRL_DOWN_MASK | MouseEvent.BUTTON2_DOWN_MASK;

        private boolean pickFixed;

        private int px;

        private int py;

        public void mousePressed(MouseEvent e) {
            final int x = e.getX();
            final int y = e.getY();
            if (createNewEdgeMode) {
                addNewEdge(x, y);
                repaint();
                return;
            }
            if (addNewNodeMode) {
                addNewNode(x, y);
                repaint();
                return;
            }
            px = x;
            py = y;
            final Node node;
            node = findNodeAt(x, y);
            if (node == null) {
                setSelectedNode(null);
                return;
            }
            if (e.getButton() == MouseEvent.BUTTON3) {
                node.toggleFixed();
                setSelectedNode(null);
            } else {
                setSelectedNode(node);
                pickFixed = node.isFixed();
                node.setFixed(true);
                //node.setPoint(x, y);
            }
            repaint();
        }

        public void mouseReleased(MouseEvent e) {
            px = -1;
            py = -1;
            final Node node = getSelectedNode();
            if (node != null) {
                node.setFixed(pickFixed);
                repaint();
            }
        }

        public void mouseDragged(MouseEvent e) {
            if (createNewEdgeMode) {
                return;
            }
            final int x = e.getX();
            final int y = e.getY();
            final Node node = getSelectedNode();
            if (node != null) {
                if (havePrevPos()) {
                    node.move(x - px, y - py);
                    repaint(DELAY);
                }
            } else if ((e.getModifiersEx() & MOVE_MASK) != 0) {
                if (havePrevPos()) {
                    moveAll(x - px, y - py);
                    repaint(DELAY);
                }
            }
            px = x;
            py = y;
        }

        private boolean havePrevPos() {
            return (px >= 0) && (py >= 0);
        }
    }
}
