package ru.ezsk.demo.graphLayout;

import java.applet.Applet;
import java.applet.AudioClip;
import java.awt.EventQueue;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JRootPane;
import javax.swing.WindowConstants;

/**
 * @version $Revision: 61 $ $Author: beholder $
 *          $Date: 2005-11-02 16:05:08 +0300 (Ср, 02 ноя 2005) $
 */
public class Launcher implements Context
{
    //========================================================== Class Methods

    public static void main(final String[] args) {
        EventQueue.invokeLater(new Runnable() {
            public void run() {
                new Launcher().run(args);
            }
        });
    }

    //================================================================= Fields

    private JFrame frame;

    private Application application;

    private Map<String, String> params = new HashMap<String, String>();

    //================================================================ Methods

    private void run(String[] args) {
        readArguments(args);
        application = new Application(this);

        frame = new JFrame("Graph Layout Demo");
        frame.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);

        frame.addWindowListener(new WindowAdapter()
        {
            public void windowClosing(WindowEvent e) {
                application.stop();
                application.destroy();
                application = null;
                frame.dispose();
                System.exit(0);
            }

            public void windowActivated(WindowEvent e) {
                application.start();
            }

            public void windowDeactivated(WindowEvent e) {
                application.stop();
            }
        });

        application.init();

        final ImageIcon icon = new ImageIcon(getClass().getResource("icon.png"));
        frame.setIconImage(icon.getImage());
        frame.setLocationByPlatform(true);
        frame.setSize(800, 600);
        frame.setVisible(true);
    }

    private void readArguments(String[] args) {
        for (int i = 0; i < args.length; i++) {
            final String arg = args[i];
            if (arg.charAt(0) == '-') {
                final String name = arg.substring(1);
                final String value;
                if (i + 1 < args.length) {
                    value = args[i + 1];
                } else {
                    value = "";
                }
                params.put(name, value);
                //noinspection AssignmentToForLoopParameter
                i++;
            }
        }
    }

    public AudioClip getAudioClip(String name) {
        final URL url = getClass().getClassLoader().getResource(name);
        return Applet.newAudioClip(url);
    }

    public String getParameter(String name) {
        return params.get(name);
    }

    public JRootPane getRootPane() {
        return frame.getRootPane();
    }

    public void showStatus(String status) {
        // todo!
        System.out.println(status);
    }
}
